"use strict";
/*!
 * @license
 * Copyright © 2005-2025 Hyland Software, Inc. and its affiliates. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const utils_1 = require("@angular-eslint/utils");
const create_eslint_rule_1 = require("../../utils/create-eslint-rule/create-eslint-rule");
exports.RULE_NAME = 'use-none-component-view-encapsulation';
const metadataPropertyName = 'encapsulation';
const viewEncapsulationNone = 'ViewEncapsulation.None';
const nodeToReport = (node) => {
    if (!utils_1.ASTUtils.isProperty(node)) {
        return node;
    }
    return utils_1.ASTUtils.isMemberExpression(node.value) ? node.value.property : node.value;
};
/**
 * Custom ESLint rule which check if component uses ViewEncapsulation.None. It has been implemented because None encapsulation makes themes styling easier.
 * It also allows to autofix.
 */
exports.default = (0, create_eslint_rule_1.createESLintRule)({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: `Disallows using other encapsulation than \`${viewEncapsulationNone}\``
        },
        hasSuggestions: true,
        schema: [],
        messages: {
            useNoneComponentViewEncapsulation: `Using encapsulation other than '${viewEncapsulationNone}' makes themes styling harder.`,
            suggestAddViewEncapsulationNone: `Add '${viewEncapsulationNone}'`
        }
    },
    defaultOptions: [],
    create(context) {
        const encapsulationProperty = utils_1.Selectors.metadataProperty(metadataPropertyName);
        const withoutEncapsulationProperty = `${utils_1.Selectors.COMPONENT_CLASS_DECORATOR}:matches([expression.arguments.length=0], [expression.arguments.0.type='ObjectExpression']:not(:has(${encapsulationProperty})))`;
        const nonNoneViewEncapsulationNoneProperty = `${utils_1.Selectors.COMPONENT_CLASS_DECORATOR} > CallExpression > ObjectExpression > ` +
            `${encapsulationProperty}:matches([value.type='Identifier'][value.name='undefined'], [value.object.name='ViewEncapsulation'][value.property.name!='None'])`;
        const selectors = [withoutEncapsulationProperty, nonNoneViewEncapsulationNoneProperty].join(',');
        return {
            [selectors](node) {
                context.report({
                    node: nodeToReport(node),
                    messageId: 'useNoneComponentViewEncapsulation',
                    suggest: [
                        {
                            messageId: 'suggestAddViewEncapsulationNone',
                            fix: (fixer) => {
                                if (utils_1.ASTUtils.isProperty(node)) {
                                    return [
                                        utils_1.RuleFixes.getImportAddFix({
                                            fixer,
                                            importName: 'ViewEncapsulation',
                                            moduleName: '@angular/core',
                                            node: node.parent
                                        }),
                                        utils_1.ASTUtils.isMemberExpression(node.value)
                                            ? fixer.replaceText(node.value.property, 'None')
                                            : fixer.replaceText(node.value, viewEncapsulationNone)
                                    ].filter(utils_1.isNotNullOrUndefined);
                                }
                                return [
                                    utils_1.RuleFixes.getImportAddFix({
                                        fixer,
                                        importName: 'ViewEncapsulation',
                                        moduleName: '@angular/core',
                                        node: node.parent
                                    }),
                                    utils_1.RuleFixes.getDecoratorPropertyAddFix(node, fixer, `${metadataPropertyName}: ${viewEncapsulationNone}`)
                                ].filter(utils_1.isNotNullOrUndefined);
                            }
                        }
                    ]
                });
            }
        };
    }
});
